@@ -3,12 +3,9 @@
 
 __all__ = ['OTSClient']
 
-import sys
-import six
 import time
-import _strptime
-
 import logging
+
 try:  # Python 2.7+
     from logging import NullHandler
 except ImportError:
@@ -21,7 +18,8 @@ try:
 except ImportError:
     import urllib.parse as urlparse
 
-from tablestore.error import *
+from tablestore.credentials import CredentialsProvider, StaticCredentialsProvider
+from tablestore.auth import SignV2, SignV4
 from tablestore.protocol import OTSProtocol
 from tablestore.connection import ConnectionPool
 from tablestore.metadata import *
@@ -37,20 +35,17 @@ class OTSClient(object):
     )，即如果某个函数有返回值，则会在描述中说明；否则返回None。
     """
 
-
     DEFAULT_ENCODING = 'utf8'
     DEFAULT_SOCKET_TIMEOUT = 50
     DEFAULT_MAX_CONNECTION = 50
     DEFAULT_LOGGER_NAME = 'tablestore-client'
 
-    protocol_class = OTSProtocol
-    connection_pool_class = ConnectionPool
-
-    def __init__(self, end_point, access_key_id, access_key_secret, instance_name, **kwargs):
+    def __init__(self, end_point, access_key_id=None, access_key_secret=None, instance_name=None,
+                 credentials_provider: CredentialsProvider = None, region: str = None, **kwargs):
         """
         初始化``OTSClient``实例。
 
-        ``end_point``是OTS服务的地址（例如 'http://instance.cn-hangzhou.ots.aliyun.com'），必须以'http://'或'https://'开头。
+        ``end_point``是OTS服务的地址（例如 'https://instance.cn-hangzhou.ots.aliyun.com'），必须以'http://'或'https://'开头。
 
         ``access_key_id``是访问OTS服务的accessid，通过官方网站申请或通过管理员获取。
 
@@ -59,6 +54,14 @@ class OTSClient(object):
         ``instance_name``是要访问的实例名，通过官方网站控制台创建或通过管理员获取。
 
         ``sts_token``是访问OTS服务的STS token，从STS服务获取，具有有效期，过期后需要重新获取。
+        
+        ``credentials_provider``是访问OTS服务的用户凭证提供者，通过该凭证提供者可以获取access_key、sts_token等参数。
+
+        ``region``是OTS服务所在的region，如果不为空则使用v4签名。
+
+        ``sign_date``使用v4签名时，签名的日期，默认为当日的UTC日期。
+
+        ``auto_update_v4_sign``使用v4签名时，是否自动更新v4签名的签名日期。
 
         ``encoding``请求参数的字符串编码类型，默认是utf8。
 
@@ -77,32 +80,36 @@ class OTSClient(object):
 
             from tablestore.client import OTSClient
 
-            client = OTSClient('your_instance_endpoint', 'your_user_id', 'your_user_key', 'your_instance_name')
+            client = OTSClient('your_instance_endpoint', 'your_user_id', 'your_user_key', 'your_instance_name', region='region')
         """
-
-        self._validate_parameter(end_point, access_key_id, access_key_secret, instance_name)
-        sts_token = kwargs.get('sts_token')
-
+        # initialize credentials provider
+        self.credentials_provider = self._create_credentials_provider(
+            end_point=end_point,
+            instance_name=instance_name,
+            access_key_id=access_key_id,
+            access_key_secret=access_key_secret,
+            sts_token=kwargs.get('sts_token'),
+            credentials_provider=credentials_provider
+        )
         self.encoding = kwargs.get('encoding')
         if self.encoding is None:
             self.encoding = OTSClient.DEFAULT_ENCODING
-
-        self.socket_timeout = kwargs.get('socket_timeout')
-        if self.socket_timeout is None:
-            self.socket_timeout = OTSClient.DEFAULT_SOCKET_TIMEOUT
-
-        self.max_connection = kwargs.get('max_connection')
-        if self.max_connection is None:
-            self.max_connection = OTSClient.DEFAULT_MAX_CONNECTION
-
-        self.ssl_version = kwargs.get('ssl_version')
+        if region is None:
+            self._signer = SignV2(self.credentials_provider, self.encoding)
+        else:
+            self._signer = SignV4(
+                self.credentials_provider,
+                self.encoding,
+                region=region,
+                **kwargs
+            )
 
         # initialize logger
         logger_name = kwargs.get('logger_name')
         if logger_name is None:
             self.logger = logging.getLogger(OTSClient.DEFAULT_LOGGER_NAME)
-            nullHandler = NullHandler()
-            self.logger.addHandler(nullHandler)
+            null_handler = NullHandler()
+            self.logger.addHandler(null_handler)
         else:
             self.logger = logging.getLogger(logger_name)
 
@@ -112,25 +119,29 @@ class OTSClient(object):
 
         if scheme != 'http' and scheme != 'https':
             raise OTSClientError(
-                "protocol of end_point must be 'http' or 'https', e.g. http://instance.cn-hangzhou.ots.aliyun.com."
+                "protocol of end_point must be 'http' or 'https', e.g. https://instance.cn-hangzhou.ots.aliyun.com."
             )
         if host == '':
             raise OTSClientError(
-                "host of end_point should be specified, e.g. http://instance.cn-hangzhou.ots.aliyun.com."
+                "host of end_point should be specified, e.g. https://instance.cn-hangzhou.ots.aliyun.com."
             )
 
-        # intialize protocol instance via user configuration
-        self.protocol = self.protocol_class(
-            access_key_id,
-            access_key_secret,
-            sts_token,
-            instance_name,
-            self.encoding,
-            self.logger
+        # initialize protocol instance via user configuration
+        self.protocol = OTSProtocol(
+            instance_name=instance_name,
+            encoding=self.encoding,
+            logger=self.logger
         )
 
         # initialize connection via user configuration
-        self.connection = self.connection_pool_class(
+        self.socket_timeout = kwargs.get('socket_timeout')
+        if self.socket_timeout is None:
+            self.socket_timeout = OTSClient.DEFAULT_SOCKET_TIMEOUT
+        self.max_connection = kwargs.get('max_connection')
+        if self.max_connection is None:
+            self.max_connection = OTSClient.DEFAULT_MAX_CONNECTION
+        self.ssl_version = kwargs.get('ssl_version')
+        self.connection = ConnectionPool(
             host, path, timeout=self.socket_timeout, maxsize=self.max_connection, client_ssl_version=self.ssl_version
         )
 
@@ -141,24 +152,18 @@ class OTSClient(object):
         self.retry_policy = retry_policy
 
     def _request_helper(self, api_name, *args, **kwargs):
-
-        query, reqheaders, reqbody = self.protocol.make_request(
-            api_name, *args, **kwargs
-        )
+        # Generate signing key, each request generate once
+        # Must generate before making request headers
+        self._signer.gen_signing_key()
+        query, req_headers, req_body = self.protocol.make_request(api_name, self._signer, *args, **kwargs)
 
         retry_times = 0
-
-        while 1:
-
+        while True:
             try:
-                status, reason, resheaders, resbody = self.connection.send_receive(
-                    query, reqheaders, reqbody
-                )
-                self.protocol.handle_error(api_name, query, status, reason, resheaders, resbody)
+                status, reason, res_headers, res_body = self.connection.send_receive(query, req_headers, req_body)
+                self.protocol.handle_error(api_name, query, status, reason, res_headers, res_body, self._signer)
                 break
-
             except OTSServiceError as e:
-
                 if self.retry_policy.should_retry(retry_times, e, api_name):
                     retry_delay = self.retry_policy.get_retry_delay(retry_times, e, api_name)
                     time.sleep(retry_delay)
@@ -166,9 +171,27 @@ class OTSClient(object):
                 else:
                     raise e
 
-        return self.protocol.parse_response(api_name, status, resheaders, resbody)
+        return self.protocol.parse_response(api_name, status, res_headers, res_body)
 
-    def create_table(self, table_meta, table_options, reserved_throughput, secondary_indexes=[]):
+    @staticmethod
+    def _create_credentials_provider(end_point, instance_name, access_key_id, access_key_secret,
+                                     sts_token: str = None,
+                                     credentials_provider: CredentialsProvider = None) -> CredentialsProvider:
+        if not isinstance(end_point, str) or end_point == '':
+            raise OTSClientError('end_point is not str or is empty.')
+        if not isinstance(instance_name, str) or instance_name == '':
+            raise OTSClientError('instance_name is not str or is empty.')
+        if credentials_provider is None:
+            if not isinstance(access_key_id, str) or access_key_id == '':
+                raise OTSClientError('access_key_id is not str or is empty.')
+            if not isinstance(access_key_secret, str) or access_key_secret == '':
+                raise OTSClientError('access_key_secret is not str or is empty.')
+            return StaticCredentialsProvider(access_key_id=access_key_id, access_key_secret=access_key_secret,
+                                             security_token=sts_token)
+        else:
+            return credentials_provider
+
+    def create_table(self, table_meta, table_options, reserved_throughput, secondary_indexes=None):
         """
         说明：根据表信息创建表。
 
@@ -191,6 +214,8 @@ class OTSClient(object):
             client.create_table(table_meta, table_options, reserved_throughput)
         """
 
+        if secondary_indexes is None:
+            secondary_indexes = []
         self._request_helper('CreateTable', table_meta, table_options, reserved_throughput, secondary_indexes)
 
     def delete_table(self, table_name):
@@ -223,7 +248,7 @@ class OTSClient(object):
 
         return self._request_helper('ListTable')
 
-    def update_table(self, table_name, table_options = None, reserved_throughput = None):
+    def update_table(self, table_name, table_options=None, reserved_throughput=None):
         """
         说明：更新表属性，目前只支持修改预留读写吞吐量。
 
@@ -243,7 +268,7 @@ class OTSClient(object):
         """
 
         return self._request_helper(
-                    'UpdateTable', table_name, table_options, reserved_throughput
+            'UpdateTable', table_name, table_options, reserved_throughput
         )
 
     def describe_table(self, table_name):
@@ -291,12 +316,12 @@ class OTSClient(object):
         """
 
         return self._request_helper(
-                    'GetRow', table_name, primary_key, columns_to_get,
-                    column_filter, max_version, time_range,
-                    start_column, end_column, token, transaction_id
+            'GetRow', table_name, primary_key, columns_to_get,
+            column_filter, max_version, time_range,
+            start_column, end_column, token, transaction_id
         )
 
-    def put_row(self, table_name, row, condition = None, return_type = None, transaction_id = None):
+    def put_row(self, table_name, row, condition=None, return_type=None, transaction_id=None):
         """
         说明：写入一行数据。返回本次操作消耗的CapacityUnit。
 
@@ -321,10 +346,10 @@ class OTSClient(object):
         """
 
         return self._request_helper(
-                    'PutRow', table_name, row, condition, return_type, transaction_id
+            'PutRow', table_name, row, condition, return_type, transaction_id
         )
 
-    def update_row(self, table_name, row, condition, return_type = None, transaction_id = None):
+    def update_row(self, table_name, row, condition, return_type=None, transaction_id=None):
         """
         说明：更新一行数据。
 
@@ -354,10 +379,10 @@ class OTSClient(object):
         """
 
         return self._request_helper(
-                    'UpdateRow', table_name, row, condition, return_type, transaction_id
+            'UpdateRow', table_name, row, condition, return_type, transaction_id
         )
 
-    def delete_row(self, table_name, row = None, condition = None, return_type = None, transaction_id = None, **kwargs):
+    def delete_row(self, table_name, row=None, condition=None, return_type=None, transaction_id=None, **kwargs):
         """
         说明：删除一行数据。
 
@@ -389,8 +414,8 @@ class OTSClient(object):
         return self._request_helper(
             'DeleteRow', table_name, primary_key, condition, return_type, transaction_id
         )
-    
-    def exe_sql_query(self,query):
+
+    def exe_sql_query(self, query):
         """
         说明：执行sql query
 
@@ -407,7 +432,7 @@ class OTSClient(object):
         row_list,table_comsume_list,search_comsume_list = client.exe_sql_query(query)
         """
         return self._request_helper(
-                    'SQLQuery', query
+            'SQLQuery', query
         )
 
     def batch_get_row(self, request):
@@ -496,7 +521,6 @@ class OTSClient(object):
 
         return BatchWriteRowResponse(request, response)
 
-
     def get_range(self, table_name, direction,
                   inclusive_start_primary_key,
                   exclusive_end_primary_key,
@@ -507,7 +531,7 @@ class OTSClient(object):
                   time_range=None,
                   start_column=None,
                   end_column=None,
-                  token = None,
+                  token=None,
                   transaction_id=None):
         """
         说明：根据范围条件获取多行数据。
@@ -545,13 +569,13 @@ class OTSClient(object):
         """
 
         return self._request_helper(
-                    'GetRange', table_name, direction,
-                    inclusive_start_primary_key, exclusive_end_primary_key,
-                    columns_to_get, limit,
-                    column_filter, max_version,
-                    time_range, start_column,
-                    end_column, token,
-                    transaction_id
+            'GetRange', table_name, direction,
+            inclusive_start_primary_key, exclusive_end_primary_key,
+            columns_to_get, limit,
+            column_filter, max_version,
+            time_range, start_column,
+            end_column, token,
+            transaction_id
         )
 
     def xget_range(self, table_name, direction,
@@ -565,7 +589,7 @@ class OTSClient(object):
                    time_range=None,
                    start_column=None,
                    end_column=None,
-                   token = None):
+                   token=None):
         """
         说明：根据范围条件获取多行数据，iterator版本。
 
@@ -634,20 +658,6 @@ class OTSClient(object):
                     if left_count <= 0:
                         return
 
-
-    def _validate_parameter(self, endpoint, access_key_id, access_key_secret, instance_name):
-        if endpoint is None or endpoint == '':
-            raise OTSClientError('endpoint is None or empty.')
-
-        if access_key_id is None or access_key_id == '':
-            raise OTSClientError('access_key_id is None or empty.')
-
-        if access_key_secret is None or access_key_secret == '':
-            raise OTSClientError('access_key_secret is None or empty.')
-
-        if instance_name is None or instance_name == '':
-            raise OTSClientError('instance_name is None or empty.')
-
     def list_search_index(self, table_name=None):
         """
         List all search indexes, or indexes under one table.
@@ -722,7 +732,6 @@ class OTSClient(object):
 
         self._request_helper('UpdateSearchIndex', table_name, index_name, index_meta)
 
-
     def describe_search_index(self, table_name, index_name):
         """
         Describe search index.
@@ -791,7 +800,7 @@ class OTSClient(object):
             compute_splits_response = client.compute_splits(table_name, index_name)
             )
         """
-        
+
         return self._request_helper('ComputeSplits', table_name, index_name)
 
     def parallel_scan(self, table_name, index_name, scan_query, session_id, columns_to_get=None):
@@ -827,7 +836,7 @@ class OTSClient(object):
             )
         """
 
-        return self._request_helper('ParallelScan', table_name, index_name, scan_query, 
+        return self._request_helper('ParallelScan', table_name, index_name, scan_query,
                                     session_id, columns_to_get)
 
     def create_secondary_index(self, table_name, index_meta, include_base_data):
@@ -902,9 +911,6 @@ class OTSClient(object):
         :type transaction_id: str
         :param transaction_id: The id of transaction.
 
-        :type key: 类型为dict
-        :param key: The partition key.
-
         Example usage:
             client.abort_transaction(transaction_id)
         """
